/*
 * DO NOT EDIT.  THIS FILE IS GENERATED FROM nsIMemory.idl
 */

#ifndef __gen_nsIMemory_h__
#define __gen_nsIMemory_h__

#include "nsISupports.h"

class nsIMemoryPressureObserver; /* forward declaration */


/* starting interface:    nsIMemory */
#define NS_IMEMORY_IID_STR "59e7e77a-38e4-11d4-8cf5-0060b0fc14a3"

#define NS_IMEMORY_IID \
  {0x59e7e77a, 0x38e4, 0x11d4, \
    { 0x8c, 0xf5, 0x00, 0x60, 0xb0, 0xfc, 0x14, 0xa3 }}

/**
 * nsIMemory: interface to allocate and deallocate memory. Also provides
 * for notifications in low-memory situations.
 */
class nsIMemory : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IMEMORY_IID)

  /**
     * Allocates a block of memory of a particular size. If the memory 
     * cannot be allocated (because of an out-of-memory condition), null
     * is returned.
     *
     * @param size - the size of the block to allocate
     * @result the block of memory
     */
  /* [noscript, notxpcom] voidPtr alloc (in size_t size); */
  NS_IMETHOD_(void *) Alloc(size_t size) = 0;

  /**
     * Reallocates a block of memory to a new size.
     *
     * @param ptr - the block of memory to reallocate
     * @param size - the new size
     * @result the reallocated block of memory
     *
     * If ptr is null, this function behaves like malloc.
     * If s is the size of the block to which ptr points, the first
     * min(s, size) bytes of ptr's block are copied to the new block.
     * If the allocation succeeds, ptr is freed and a pointer to the 
     * new block returned.  If the allocation fails, ptr is not freed
     * and null is returned. The returned value may be the same as ptr.
     */
  /* [noscript, notxpcom] voidPtr realloc (in voidPtr ptr, in size_t newSize); */
  NS_IMETHOD_(void *) Realloc(void * ptr, size_t newSize) = 0;

  /**
     * Frees a block of memory. Null is a permissible value, in which case
     * nothing happens. 
     *
     * @param ptr - the block of memory to free
     */
  /* [noscript, notxpcom] void free (in voidPtr ptr); */
  NS_IMETHOD_(void) Free(void * ptr) = 0;

  /**
     * Attempts to shrink the heap.
     */
  /* void heapMinimize (); */
  NS_IMETHOD HeapMinimize(void) = 0;

  /**
     * Allows a memory pressure observer to be registered to provide a
     * callback for low-memory situations, allowing unnecessary objects to
     * be freed (e.g. flushing caches). Observers will be called either
     * before alloc or realloc are about to return null, or when
     * heapMinimize is called.
     * @param obs - the observer to register
     */
  /* void registerObserver (in nsIMemoryPressureObserver obs); */
  NS_IMETHOD RegisterObserver(nsIMemoryPressureObserver *obs) = 0;

  /**
     * Allows a memory pressure observer to be unregistered.
     * @param obs - the observer to unregister
     */
  /* void unregisterObserver (in nsIMemoryPressureObserver obs); */
  NS_IMETHOD UnregisterObserver(nsIMemoryPressureObserver *obs) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIMEMORY \
  NS_IMETHOD_(void *) Alloc(size_t size); \
  NS_IMETHOD_(void *) Realloc(void * ptr, size_t newSize); \
  NS_IMETHOD_(void) Free(void * ptr); \
  NS_IMETHOD HeapMinimize(void); \
  NS_IMETHOD RegisterObserver(nsIMemoryPressureObserver *obs); \
  NS_IMETHOD UnregisterObserver(nsIMemoryPressureObserver *obs); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIMEMORY(_to) \
  NS_IMETHOD_(void *) Alloc(size_t size) { return _to ## Alloc(size); } \
  NS_IMETHOD_(void *) Realloc(void * ptr, size_t newSize) { return _to ## Realloc(ptr, newSize); } \
  NS_IMETHOD_(void) Free(void * ptr) { return _to ## Free(ptr); } \
  NS_IMETHOD HeapMinimize(void) { return _to ## HeapMinimize(); } \
  NS_IMETHOD RegisterObserver(nsIMemoryPressureObserver *obs) { return _to ## RegisterObserver(obs); } \
  NS_IMETHOD UnregisterObserver(nsIMemoryPressureObserver *obs) { return _to ## UnregisterObserver(obs); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsMemory : public nsIMemory
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIMEMORY

  nsMemory();
  virtual ~nsMemory();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsMemory, nsIMemory)

nsMemory::nsMemory()
{
  NS_INIT_ISUPPORTS();
  /* member initializers and constructor code */
}

nsMemory::~nsMemory()
{
  /* destructor code */
}

/* [noscript, notxpcom] voidPtr alloc (in size_t size); */
NS_IMETHODIMP_(void *) nsMemory::Alloc(size_t size)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript, notxpcom] voidPtr realloc (in voidPtr ptr, in size_t newSize); */
NS_IMETHODIMP_(void *) nsMemory::Realloc(void * ptr, size_t newSize)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* [noscript, notxpcom] void free (in voidPtr ptr); */
NS_IMETHODIMP_(void) nsMemory::Free(void * ptr)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void heapMinimize (); */
NS_IMETHODIMP nsMemory::HeapMinimize()
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void registerObserver (in nsIMemoryPressureObserver obs); */
NS_IMETHODIMP nsMemory::RegisterObserver(nsIMemoryPressureObserver *obs)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* void unregisterObserver (in nsIMemoryPressureObserver obs); */
NS_IMETHODIMP nsMemory::UnregisterObserver(nsIMemoryPressureObserver *obs)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


/* starting interface:    nsIMemoryPressureObserver */
#define NS_IMEMORYPRESSUREOBSERVER_IID_STR "fe6c8bd4-38e4-11d4-8cf5-0060b0fc14a3"

#define NS_IMEMORYPRESSUREOBSERVER_IID \
  {0xfe6c8bd4, 0x38e4, 0x11d4, \
    { 0x8c, 0xf5, 0x00, 0x60, 0xb0, 0xfc, 0x14, 0xa3 }}

class nsIMemoryPressureObserver : public nsISupports {
 public: 

  NS_DEFINE_STATIC_IID_ACCESSOR(NS_IMEMORYPRESSUREOBSERVER_IID)

  /**
 * nsIMemoryPressureObserver: interface for low-memory notification.
 */
  enum { REASON_ALLOC_FAILURE = 0U };

  enum { REASON_HEAP_MINIMIZE = 1U };

  /**
     * Called in response to a low-memory condition. 
     * @param reason - either REASON_ALLOC_FAILURE when alloc or realloc 
     *                 fails, or REASON_HEAP_MINIMIZE when heapMinimize is
     *                 called
     * @param requestedAmount - either the size requested by alloc or realloc,
     *                 or 0 if heapMinimize was called
     */
  /* void flushMemory (in unsigned long reason, in size_t requestedAmount); */
  NS_IMETHOD FlushMemory(PRUint32 reason, size_t requestedAmount) = 0;

};

/* Use this macro when declaring classes that implement this interface. */
#define NS_DECL_NSIMEMORYPRESSUREOBSERVER \
  NS_IMETHOD FlushMemory(PRUint32 reason, size_t requestedAmount); 

/* Use this macro to declare functions that forward the behavior of this interface to another object. */
#define NS_FORWARD_NSIMEMORYPRESSUREOBSERVER(_to) \
  NS_IMETHOD FlushMemory(PRUint32 reason, size_t requestedAmount) { return _to ## FlushMemory(reason, requestedAmount); } 

#if 0
/* Use the code below as a template for the implementation class for this interface. */

/* Header file */
class nsMemoryPressureObserver : public nsIMemoryPressureObserver
{
public:
  NS_DECL_ISUPPORTS
  NS_DECL_NSIMEMORYPRESSUREOBSERVER

  nsMemoryPressureObserver();
  virtual ~nsMemoryPressureObserver();
  /* additional members */
};

/* Implementation file */
NS_IMPL_ISUPPORTS1(nsMemoryPressureObserver, nsIMemoryPressureObserver)

nsMemoryPressureObserver::nsMemoryPressureObserver()
{
  NS_INIT_ISUPPORTS();
  /* member initializers and constructor code */
}

nsMemoryPressureObserver::~nsMemoryPressureObserver()
{
  /* destructor code */
}

/* void flushMemory (in unsigned long reason, in size_t requestedAmount); */
NS_IMETHODIMP nsMemoryPressureObserver::FlushMemory(PRUint32 reason, size_t requestedAmount)
{
    return NS_ERROR_NOT_IMPLEMENTED;
}

/* End of implementation class template. */
#endif


#endif /* __gen_nsIMemory_h__ */
